[llvm-commits] [llvm] r158937 - in /llvm/trunk: include/llvm/Analysis/MemoryBuiltins.h lib/Analysis/MemoryBuiltins.cpp lib/Transforms/InstCombine/InstCombineCalls.cpp lib/Transforms/InstCombine/InstructionCombining.cpp test/Transforms/InstCombine/objsize-64.ll

Nuno Lopes nunoplopes at sapo.pt
Thu Jun 21 14:25:05 PDT 2012


Author: nlopes
Date: Thu Jun 21 16:25:05 2012
New Revision: 158937

URL: http://llvm.org/viewvc/llvm-project?rev=158937&view=rev
Log:
Add support for invoke to the MemoryBuiltin analysid.
Update comments accordingly.

Make instcombine remove useless invokes to C++'s 'new' allocation function (test attached).

Modified:
    llvm/trunk/include/llvm/Analysis/MemoryBuiltins.h
    llvm/trunk/lib/Analysis/MemoryBuiltins.cpp
    llvm/trunk/lib/Transforms/InstCombine/InstCombineCalls.cpp
    llvm/trunk/lib/Transforms/InstCombine/InstructionCombining.cpp
    llvm/trunk/test/Transforms/InstCombine/objsize-64.ll

Modified: llvm/trunk/include/llvm/Analysis/MemoryBuiltins.h
URL: http://llvm.org/viewvc/llvm-project/llvm/trunk/include/llvm/Analysis/MemoryBuiltins.h?rev=158937&r1=158936&r2=158937&view=diff
==============================================================================
--- llvm/trunk/include/llvm/Analysis/MemoryBuiltins.h (original)
+++ llvm/trunk/include/llvm/Analysis/MemoryBuiltins.h Thu Jun 21 16:25:05 2012
@@ -31,28 +31,29 @@
 class Value;
 
 
-/// \brief Tests if a value is a call to a library function that allocates or
-/// reallocates memory (either malloc, calloc, realloc, or strdup like).
+/// \brief Tests if a value is a call or invoke to a library function that
+/// allocates or reallocates memory (either malloc, calloc, realloc, or strdup
+/// like).
 bool isAllocationFn(const Value *V, bool LookThroughBitCast = false);
 
-/// \brief Tests if a value is a call to a function that returns a NoAlias
-/// pointer (including malloc/calloc/strdup-like functions).
+/// \brief Tests if a value is a call or invoke to a function that returns a
+/// NoAlias pointer (including malloc/calloc/strdup-like functions).
 bool isNoAliasFn(const Value *V, bool LookThroughBitCast = false);
 
-/// \brief Tests if a value is a call to a library function that allocates
-/// uninitialized memory (such as malloc).
+/// \brief Tests if a value is a call or invoke to a library function that
+/// allocates uninitialized memory (such as malloc).
 bool isMallocLikeFn(const Value *V, bool LookThroughBitCast = false);
 
-/// \brief Tests if a value is a call to a library function that allocates
-/// zero-filled memory (such as calloc).
+/// \brief Tests if a value is a call or invoke to a library function that
+/// allocates zero-filled memory (such as calloc).
 bool isCallocLikeFn(const Value *V, bool LookThroughBitCast = false);
 
-/// \brief Tests if a value is a call to a library function that allocates
-/// memory (either malloc, calloc, or strdup like).
+/// \brief Tests if a value is a call or invoke to a library function that
+/// allocates memory (either malloc, calloc, or strdup like).
 bool isAllocLikeFn(const Value *V, bool LookThroughBitCast = false);
 
-/// \brief Tests if a value is a call to a library function that reallocates
-/// memory (such as realloc).
+/// \brief Tests if a value is a call or invoke to a library function that
+/// reallocates memory (such as realloc).
 bool isReallocLikeFn(const Value *V, bool LookThroughBitCast = false);
 
 

Modified: llvm/trunk/lib/Analysis/MemoryBuiltins.cpp
URL: http://llvm.org/viewvc/llvm-project/llvm/trunk/lib/Analysis/MemoryBuiltins.cpp?rev=158937&r1=158936&r2=158937&view=diff
==============================================================================
--- llvm/trunk/lib/Transforms/InstCombine/InstCombineCalls.cpp (original)
+++ llvm/trunk/lib/Transforms/InstCombine/InstCombineCalls.cpp Thu Jun 21 16:25:05 2012
@@ -172,8 +172,6 @@
 Instruction *InstCombiner::visitCallInst(CallInst &CI) {
   if (isFreeCall(&CI))
     return visitFree(CI);
-  if (isAllocLikeFn(&CI))
-    return visitMalloc(CI);
 
   // If the caller function is nounwind, mark the call as nounwind, even if the
   // callee isn't.
@@ -881,6 +879,9 @@
 // visitCallSite - Improvements for call and invoke instructions.
 //
 Instruction *InstCombiner::visitCallSite(CallSite CS) {
+  if (isAllocLikeFn(CS.getInstruction()))
+    return visitMalloc(*CS.getInstruction());
+
   bool Changed = false;
 
   // If the callee is a pointer to a function, attempt to move any casts to the

Modified: llvm/trunk/lib/Transforms/InstCombine/InstructionCombining.cpp
URL: http://llvm.org/viewvc/llvm-project/llvm/trunk/lib/Transforms/InstCombine/InstructionCombining.cpp?rev=158937&r1=158936&r2=158937&view=diff
==============================================================================
--- llvm/trunk/lib/Transforms/InstCombine/InstructionCombining.cpp (original)
+++ llvm/trunk/lib/Transforms/InstCombine/InstructionCombining.cpp Thu Jun 21 16:25:05 2012
@@ -1167,6 +1167,10 @@
       }
       EraseInstFromFunction(*I);
     }
+
+    if (InvokeInst *II = dyn_cast<InvokeInst>(&MI)) {
+      BranchInst::Create(II->getNormalDest(), II->getParent());
+    }
     return EraseInstFromFunction(MI);
   }
   return 0;

Modified: llvm/trunk/test/Transforms/InstCombine/objsize-64.ll
URL: http://llvm.org/viewvc/llvm-project/llvm/trunk/test/Transforms/InstCombine/objsize-64.ll?rev=158937&r1=158936&r2=158937&view=diff
==============================================================================
--- llvm/trunk/test/Transforms/InstCombine/objsize-64.ll (original)
+++ llvm/trunk/test/Transforms/InstCombine/objsize-64.ll Thu Jun 21 16:25:05 2012
@@ -2,6 +2,9 @@
 target datalayout = "e-p:64:64:64-i1:8:8-i8:8:8-i16:16:16-i32:32:32-i64:64:64-f32:32:32-f64:64:64-v64:64:64-v128:128:128-a0:0:64-s0:64:64-f80:128:128-n8:16:32:64-S128"
 
 declare noalias i8* @malloc(i32) nounwind
+declare noalias i8* @_Znwm(i64)  ; new(unsigned long)
+declare i32 @__gxx_personality_v0(...)
+declare void @__cxa_call_unexpected(i8*)
 declare i64 @llvm.objectsize.i64(i8*, i1) nounwind readonly
 
 ; CHECK: @f1
@@ -11,3 +14,24 @@
 ; CHECK-NEXT: ret i64 4
   ret i64 %size
 }
+
+
+; CHECK: @f2
+define i64 @f2() nounwind uwtable ssp {
+entry:
+; CHECK: br label
+  %call = invoke noalias i8* @_Znwm(i64 13)
+          to label %invoke.cont unwind label %lpad
+
+invoke.cont:
+; CHECK: ret i64 13
+  %0 = tail call i64 @llvm.objectsize.i64(i8* %call, i1 false)
+  ret i64 %0
+
+lpad:
+  %1 = landingpad { i8*, i32 } personality i8* bitcast (i32 (...)* @__gxx_personality_v0 to i8*)
+          filter [0 x i8*] zeroinitializer
+  %2 = extractvalue { i8*, i32 } %1, 0
+  tail call void @__cxa_call_unexpected(i8* %2) noreturn nounwind
+  unreachable
+}





More information about the llvm-commits mailing list
--- llvm/trunk/lib/Analysis/MemoryBuiltins.cpp (original)
+++ llvm/trunk/lib/Analysis/MemoryBuiltins.cpp Thu Jun 21 16:25:05 2012
@@ -65,11 +65,17 @@
 static Function *getCalledFunction(const Value *V, bool LookThroughBitCast) {
   if (LookThroughBitCast)
     V = V->stripPointerCasts();
-  const CallInst *CI = dyn_cast<CallInst>(V);
-  if (!CI)
+
+  Value *I = const_cast<Value*>(V);
+  CallSite CS;
+  if (CallInst *CI = dyn_cast<CallInst>(I))
+    CS = CallSite(CI);
+  else if (InvokeInst *II = dyn_cast<InvokeInst>(I))
+    CS = CallSite(II);
+  else
     return 0;
 
-  Function *Callee = CI->getCalledFunction();
+  Function *Callee = CS.getCalledFunction();
   if (!Callee || !Callee->isDeclaration())
     return 0;
   return Callee;
@@ -122,39 +128,40 @@
 }
 
 
 /// \brief Tests if a value is a call to a library function that allocates or
 /// reallocates memory (either malloc, calloc, realloc, or strdup like).
 bool llvm::isAllocationFn(const Value *V, bool LookThroughBitCast) {
   return getAllocationData(V, AnyAlloc, LookThroughBitCast);
 }
 
 /// \brief Tests if a value is a call to a function that returns a NoAlias
 /// pointer (including malloc/calloc/strdup-like functions).
 bool llvm::isNoAliasFn(const Value *V, bool LookThroughBitCast) {
   return isAllocLikeFn(V, LookThroughBitCast) ||
          hasNoAliasAttr(V, LookThroughBitCast);
 }
 
 /// \brief Tests if a value is a call to a library function that allocates
 /// uninitialized memory (such as malloc).
 bool llvm::isMallocLikeFn(const Value *V, bool LookThroughBitCast) {
   return getAllocationData(V, MallocLike, LookThroughBitCast);
 }
 
 /// \brief Tests if a value is a call to a library function that allocates
 /// zero-filled memory (such as calloc).
 bool llvm::isCallocLikeFn(const Value *V, bool LookThroughBitCast) {
   return getAllocationData(V, CallocLike, LookThroughBitCast);
 }
 
 /// \brief Tests if a value is a call to a library function that allocates
 /// memory (either malloc, calloc, or strdup like).
 bool llvm::isAllocLikeFn(const Value *V, bool LookThroughBitCast) {
   return getAllocationData(V, AllocLike, LookThroughBitCast);
 }
 
 /// \brief Tests if a value is a call to a library function that reallocates
 /// memory (such as realloc).
 bool llvm::isReallocLikeFn(const Value *V, bool LookThroughBitCast) {
   return getAllocationData(V, ReallocLike, LookThroughBitCast);
 }
@@ -122,39 +128,40 @@
 
 
-/// \brief Tests if a value is a call to a library function that allocates or
-/// reallocates memory (either malloc, calloc, realloc, or strdup like).
+/// \brief Tests if a value is a call or invoke to a library function that
+/// allocates or reallocates memory (either malloc, calloc, realloc, or strdup
+/// like).
 bool llvm::isAllocationFn(const Value *V, bool LookThroughBitCast) {
   return getAllocationData(V, AnyAlloc, LookThroughBitCast);
@@ -122,39 +128,40 @@
 }
 
-/// \brief Tests if a value is a call to a function that returns a NoAlias
-/// pointer (including malloc/calloc/strdup-like functions).
+/// \brief Tests if a value is a call or invoke to a function that returns a
+/// NoAlias pointer (including malloc/calloc/strdup-like functions).
 bool llvm::isNoAliasFn(const Value *V, bool LookThroughBitCast) {
   return isAllocLikeFn(V, LookThroughBitCast) ||
@@ -122,39 +128,40 @@
 }
 
-/// \brief Tests if a value is a call to a library function that allocates
-/// uninitialized memory (such as malloc).
+/// \brief Tests if a value is a call or invoke to a library function that
+/// allocates uninitialized memory (such as malloc).
 bool llvm::isMallocLikeFn(const Value *V, bool LookThroughBitCast) {
   return getAllocationData(V, MallocLike, LookThroughBitCast);
@@ -122,39 +128,40 @@
 }
 
-/// \brief Tests if a value is a call to a library function that allocates
-/// zero-filled memory (such as calloc).
+/// \brief Tests if a value is a call or invoke to a library function that
+/// allocates zero-filled memory (such as calloc).
 bool llvm::isCallocLikeFn(const Value *V, bool LookThroughBitCast) {
   return getAllocationData(V, CallocLike, LookThroughBitCast);
@@ -122,39 +128,40 @@
 }
 
-/// \brief Tests if a value is a call to a library function that allocates
-/// memory (either malloc, calloc, or strdup like).
+/// \brief Tests if a value is a call or invoke to a library function that
+/// allocates memory (either malloc, calloc, or strdup like).
 bool llvm::isAllocLikeFn(const Value *V, bool LookThroughBitCast) {
   return getAllocationData(V, AllocLike, LookThroughBitCast);
@@ -122,39 +128,40 @@
 }
 
-/// \brief Tests if a value is a call to a library function that reallocates
-/// memory (such as realloc).
+/// \brief Tests if a value is a call or invoke to a library function that
+/// reallocates memory (such as realloc).
 bool llvm::isReallocLikeFn(const Value *V, bool LookThroughBitCast) {
   return getAllocationData(V, ReallocLike, LookThroughBitCast);

Modified: llvm/trunk/lib/Transforms/InstCombine/InstCombineCalls.cpp
URL: http://llvm.org/viewvc/llvm-project/llvm/trunk/lib/Transforms/InstCombine/InstCombineCalls.cpp?rev=158937&r1=158936&r2=158937&view=diff
==============================================================================